import { system, world, BlockPermutation, Direction } from '@minecraft/server';
import { decrementStack, getOppositeDirection, DirectionType, cardinalSides } from './utils/helper';
import { directionToVector3 } from './utils/math';

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.blockComponentRegistry.registerCustomComponent('pa_lavender_flower:trigger', {
  onRandomTick: e => { const growthChance = 1 / 3;
if (Math.random() > growthChance) return;

const growth = e.block.permutation.getState("pa:growth");
e.block.setPermutation(
    e.block.permutation.withState("pa:growth", growth + 1)
);
 },
onPlayerInteract: e => { if (!e.player) return;

const equippable = e.player.getComponent("minecraft:equippable");
if (!equippable) return;

const mainhand = equippable.getEquipmentSlot("Mainhand");
if (!mainhand.hasItem() || mainhand.typeId !== "minecraft:bone_meal")
    return;

let growth = e.block.permutation.getState("pa:growth");

if (growth >= 2) return;

growth += 1;
e.block.setPermutation(
    e.block.permutation.withState("pa:growth", growth)
);

if (mainhand.amount > 1) mainhand.amount--;
else mainhand.setItem(undefined);

const effectLocation = e.block.center();
e.dimension.playSound("item.bone_meal.use", effectLocation);
e.dimension.spawnParticle(
    "minecraft:crop_growth_emitter",
    effectLocation
);
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('pa_cherry_crop:trigger', {
  onRandomTick: e => { const growthChance = 1 / 3;
if (Math.random() > growthChance) return;

const growth = e.block.permutation.getState("pa:growth");
e.block.setPermutation(
    e.block.permutation.withState("pa:growth", growth + 1)
);
 },
onPlayerInteract: e => { if (!e.player) return;

const equippable = e.player.getComponent("minecraft:equippable");
if (!equippable) return;

const mainhand = equippable.getEquipmentSlot("Mainhand");
if (!mainhand.hasItem() || mainhand.typeId !== "minecraft:bone_meal")
    return;

let growth = e.block.permutation.getState("pa:growth");

if (growth >= 4) return;

growth += 1;
e.block.setPermutation(
    e.block.permutation.withState("pa:growth", growth)
);

if (mainhand.amount > 1) mainhand.amount--;
else mainhand.setItem(undefined);

const effectLocation = e.block.center();
e.dimension.playSound("item.bone_meal.use", effectLocation);
e.dimension.spawnParticle(
    "minecraft:crop_growth_emitter",
    effectLocation
);
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('pa_jc_fence_gate:trigger', {
  onPlayerInteract: e => { let face = getOppositeDirection(e.face);

const isOpen = e.block.permutation.getState("pa:open");
const sound = isOpen ? "cherry_wood" : "cherry_wood";

e.block.setPermutation(e.block.permutation.withState("minecraft:cardinal_direction", face.toLowerCase()));
e.block.setPermutation(e.block.permutation.withState("pa:open", !isOpen));

e.dimension.playSound(sound, e.block.center(), {
    pitch: 0.9,
    volume: 0.9,
});
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('pa_jc_slab:trigger', {
  beforeOnPlayerPlace: e => { let playerEquipment = e.player.getComponent('equippable').getEquipment('Mainhand');
if (playerEquipment === undefined)
    return;
let face = getOppositeDirection(e.face);
let blockToCheck = e.block.offset(directionToVector3(face));
const blockStateDouble = 'pa:is_double';
const blockStateHalf = 'minecraft:vertical_half';
if (blockToCheck.permutation.getState(blockStateDouble))
    return;
if (blockToCheck.typeId !== playerEquipment.typeId)
    return;
if (face === 'Up' || face === 'Down') {
    const state = blockToCheck.permutation.getState(blockStateHalf);
    if ((face === 'Up' && state === 'top') ||
        (face === 'Down' && state === 'bottom')) {
        blockToCheck.dimension.playSound("use.cherry_wood", blockToCheck.location, { pitch: 0.8, volume: 1.0 });
        blockToCheck.setPermutation(BlockPermutation.resolve(blockToCheck.typeId, {
            [blockStateDouble]: true
        }));
        e.cancel = true;
        decrementStack(player);
    }
}
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('pa_jc_fence:trigger', {
  onPlayerDestroy: e => { for (const direction of DirectionType.HORIZONTAL) {
    const directionOpposite = getOppositeDirection(direction);
    const blockToCheck = e.block.offset(directionToVector3(directionOpposite));

    for (const directionChild of DirectionType.HORIZONTAL) {
        const directionOppositeChild = getOppositeDirection(directionChild);
        const blockToCheckChild = blockToCheck.offset(directionToVector3(directionOppositeChild));

        try {
            blockToCheck.setPermutation(blockToCheck.permutation.withState(`pa:${directionOppositeChild.toLowerCase()}`, !blockToCheckChild.typeId.includes("minecraft:air")));
        } catch (error) { }
    }
}
 },
beforeOnPlayerPlace: e => { e.permutationToPlace = e.permutationToPlace.withState("pa:in_world", true);

for (const direction of DirectionType.HORIZONTAL) {
    const directionOpposite = getOppositeDirection(direction);
    const blockToCheck = e.block.offset(directionToVector3(directionOpposite));

    if (blockToCheck.typeId.includes("minecraft:air")) continue;
    e.permutationToPlace = e.permutationToPlace.withState(`pa:${directionOpposite.toLowerCase()}`, !blockToCheck.typeId.includes("minecraft:air"));
    try {
        blockToCheck.setPermutation(blockToCheck.permutation.withState(`pa:${direction.toLowerCase()}`, true));
    } catch (error) { }
}
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('pa_jc_stair:trigger', {
  beforeOnPlayerPlace: e => { let face = e.permutationToPlace.getState('minecraft:cardinal_direction');
let half = e.permutationToPlace.getState('minecraft:vertical_half');
let blockPermutation = e.permutationToPlace;
let blockToCheck;
switch (face) {
    case 'north':
        {
            for (const direction of DirectionType.HORIZONTAL) {
                blockToCheck = e.block.offset(directionToVector3(direction));
                if (blockToCheck.typeId !== blockPermutation.type.id)
                    continue;
                const blockToCheckHalf = blockToCheck.permutation.getState('minecraft:vertical_half');
                if (blockToCheckHalf !== half)
                    continue;
                const blockToCheckDirection = blockToCheck.permutation.getState('minecraft:cardinal_direction');
                switch (direction) {
                    case Direction.North:
                        {
                            if (blockToCheckDirection === 'west') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_left');
                            }
                            if (blockToCheckDirection === 'east') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_right');
                            }
                        }
                        break;
                    case Direction.South:
                        {
                            if (blockToCheckDirection === 'east') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_left');
                            }
                            if (blockToCheckDirection === 'west') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_right');
                            }
                        }
                        break;
                    case Direction.East:
                        {
                            if (blockToCheckDirection === 'east') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_right'));
                            }
                            if (blockToCheckDirection === 'west') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_right'));
                            }
                        }
                        break;
                    case Direction.West:
                        {
                            if (blockToCheckDirection === 'west') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_left'));
                            }
                            if (blockToCheckDirection === 'east') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_left'));
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        break;
    case 'south':
        {
            blockPermutation = blockPermutation.withState('pa:south', true);
            for (const direction of DirectionType.HORIZONTAL) {
                blockToCheck = e.block.offset(directionToVector3(direction));
                if (blockToCheck.typeId !== blockPermutation.type.id)
                    continue;
                const blockToCheckHalf = blockToCheck.permutation.getState('minecraft:vertical_half');
                if (blockToCheckHalf !== half)
                    continue;
                const blockToCheckDirection = blockToCheck.permutation.getState('minecraft:cardinal_direction');
                switch (direction) {
                    case Direction.North:
                        {
                            if (blockToCheckDirection === 'west') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_left');
                            }
                            if (blockToCheckDirection === 'east') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_right');
                            }
                        }
                        break;
                    case Direction.South:
                        {
                            if (blockToCheckDirection === 'east') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_left');
                            }
                            if (blockToCheckDirection === 'west') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_right');
                            }
                        }
                        break;
                    case Direction.East:
                        {
                            if (blockToCheckDirection === 'east') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_left'));
                            }
                            if (blockToCheckDirection === 'west') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_left'));
                            }
                        }
                        break;
                    case Direction.West:
                        {
                            if (blockToCheckDirection === 'west') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_right'));
                            }
                            if (blockToCheckDirection === 'east') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_right'));
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        break;
    case 'east':
        {
            for (const direction of DirectionType.HORIZONTAL) {
                blockToCheck = e.block.offset(directionToVector3(direction));
                if (blockToCheck.typeId !== blockPermutation.type.id)
                    continue;
                const blockToCheckHalf = blockToCheck.permutation.getState('minecraft:vertical_half');
                if (blockToCheckHalf !== half)
                    continue;
                const blockToCheckDirection = blockToCheck.permutation.getState('minecraft:cardinal_direction');
                switch (direction) {
                    case Direction.East:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_left');
                            }
                            if (blockToCheckDirection === 'south') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_right');
                            }
                        }
                        break;
                    case Direction.West:
                        {
                            if (blockToCheckDirection === 'south') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_left');
                            }
                            if (blockToCheckDirection === 'north') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_right');
                            }
                        }
                        break;
                    case Direction.North:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_left'));
                            }
                            if (blockToCheckDirection === 'south') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_left'));
                            }
                        }
                        break;
                    case Direction.South:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_right'));
                            }
                            if (blockToCheckDirection === 'south') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_right'));
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        break;
    case 'west':
        {
            for (const direction of DirectionType.HORIZONTAL) {
                blockToCheck = e.block.offset(directionToVector3(direction));
                if (blockToCheck.typeId !== blockPermutation.type.id)
                    continue;
                const blockToCheckHalf = blockToCheck.permutation.getState('minecraft:vertical_half');
                if (blockToCheckHalf !== half)
                    continue;
                const blockToCheckDirection = blockToCheck.permutation.getState('minecraft:cardinal_direction');
                switch (direction) {
                    case Direction.East:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_left');
                            }
                            if (blockToCheckDirection === 'south') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'inner_right');
                            }
                        }
                        break;
                    case Direction.West:
                        {
                            if (blockToCheckDirection === 'south') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_left');
                            }
                            if (blockToCheckDirection === 'north') {
                                blockPermutation =
                                    blockPermutation.withState('pa:shape', 'outer_right');
                            }
                        }
                        break;
                    case Direction.North:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_right'));
                            }
                            if (blockToCheckDirection === 'south') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_right'));
                            }
                        }
                        break;
                    case Direction.South:
                        {
                            if (blockToCheckDirection === 'north') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'outer_left'));
                            }
                            if (blockToCheckDirection === 'south') {
                                blockToCheck.setPermutation(blockToCheck.permutation.withState('pa:shape', 'inner_left'));
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        break;
    default:
        break;
}
e.permutationToPlace = blockPermutation;
e.block.dimension.playSound("use.cherry_wood", e.block.location, { pitch: 0.8, volume: 1.0 });
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('pa_jc_trapdoor:trigger', {
  onPlayerInteract: e => { const isOpen = e.block.permutation.getState("pa:open");
const sound = isOpen ? "cherry_wood" : "cherry_wood";

e.block.setPermutation(e.block.permutation.withState("pa:open", !isOpen));

e.dimension.playSound(sound, e.block.center(), {
  pitch: 0.9,
  volume: 0.9,
});
 },
});

initEvent.blockComponentRegistry.registerCustomComponent('pa_jc_door:trigger', {
  onPlayerInteract: e => { const { block } = e;
const isOpen = block.permutation.getState('pa:is_open');

const getHalfDoor = () => {
  switch (block.permutation.getState('pa:part')) {
    case 'lower': return block.above();
    case 'upper': return block.below();
  }
}
const halfDoor = getHalfDoor();

block.setPermutation(block.permutation.withState('pa:is_open', !isOpen));
block.dimension.playSound(isOpen ? 'cherry_wood' : 'cherry_wood', block.location, { pitch: 1.0, volume: 0.4 });
if (halfDoor?.permutation.matches(block.typeId)) {
  halfDoor.setPermutation(halfDoor.permutation.withState('pa:is_open', !isOpen));
}
 },
beforeOnPlayerPlace: e => { const { block, permutationToPlace } = e;

let face = getOppositeDirection(e.face);

const aboveBlock = block.above();
const canBePlace = aboveBlock.isAir || aboveBlock.isLiquid || face === 'Down';

if (canBePlace) {
  const hasSideDoor = (sideBlock) => {
    return sideBlock.matches(permutationToPlace.type.id, {
      ...sideBlock.permutation.getAllStates(),
      'pa:part': 'lower'
    });
  }

  const cardinalDirection = permutationToPlace.getState('minecraft:cardinal_direction');
  const sides = cardinalSides[cardinalDirection];
  const leftBlock = block[sides.left]();
  const rightBlock = block[sides.right]();
  const isSideDoor = hasSideDoor(leftBlock) && !hasSideDoor(rightBlock);
  const hasRightBlock = rightBlock.isSolid || rightBlock.above().isSolid;
  const doorStates = {
    'minecraft:cardinal_direction': cardinalDirection,
    'pa:side': isSideDoor || hasRightBlock ? 'right' : 'left',
  }

  const doorPermutation = BlockPermutation.resolve(permutationToPlace.type.id, doorStates);

  e.permutationToPlace = doorPermutation.withState('pa:part', 'lower');
  aboveBlock.setPermutation(doorPermutation.withState('pa:part', 'upper'));
}
else {
  e.cancel = true;
}
 },
});

});
